@@ -1,7 +1,8 @@
+use super::errors::FxError;
+use super::state::FX_CONFIG_DIR;
 use serde::{Deserialize, Serialize};
 use std::fs::read_to_string;
-use super::errors::MyError;
-use super::state::FX_CONFIG_DIR;
+use std::path::Path;
 
 pub const SESSION_FILE: &str = ".session";
 pub const SESSION_EXAMPLE: &str = "sort_by = \"Name\"
@@ -11,16 +12,16 @@ show_hidden = false
 #[derive(Deserialize, Serialize, Debug, Clone)]
 pub struct Session {
     pub sort_by: SortKey,
-    pub show_hidden: bool
+    pub show_hidden: bool,
 }
 
-#[derive(Deserialize,Serialize, Debug, Clone)]
+#[derive(Deserialize, Serialize, Debug, Clone)]
 pub enum SortKey {
     Name,
     Time,
 }
 
-pub fn read_session() -> Result<Session, MyError> {
+pub fn read_session() -> Result<Session, FxError> {
     let mut session = dirs::config_dir().unwrap_or_else(|| panic!("Cannot read config dir."));
     session.push(FX_CONFIG_DIR);
     session.push(SESSION_FILE);
@@ -31,4 +32,10 @@ pub fn read_session() -> Result<Session, MyError> {
     } else {
         panic!("Cannot deserialize session file.");
     }
-}
\ No newline at end of file
+}
+
+pub fn make_session(session_file: &Path) -> Result<(), FxError> {
+    std::fs::write(&session_file, SESSION_EXAMPLE)
+        .unwrap_or_else(|_| panic!("Cannot write new session file."));
+    Ok(())
+}
